import 'dart:async';

import 'package:analyzer/dart/element/element.dart';
import 'package:b_annotations/bloc.dart';
import 'package:b_generator/src/helpers/string_helpers.dart';
import 'package:build/build.dart';
import 'package:source_gen/source_gen.dart';

import 'helpers/annotated_method_visitor.dart';

class BlocGenerator extends GeneratorForAnnotation<BlocAnnotation> {
  @override
  FutureOr<String> generateForAnnotatedElement(Element element, ConstantReader annotation, BuildStep buildStep) {
		// @formatter:off
		final String stateName = annotation.peek('state')?.typeValue?.getDisplayString(withNullability: false) ?? "${element.name}State";
		final Set<MethodElement> events = {};
		element.visitChildren(AnnotatedMethodVisitor<EventAnnotation>((method) => events.add(method)));
		// @formatter:on

		return """
    abstract class GeneratedBloc extends Bloc<${eventName(element)}, $stateName> {
      $stateName get initialState;
      
      ${mapEventsToState(element, events, stateName)}
      
      ${events.map((event) => mapEventToState(element, event, stateName)).join('\n')}
    }""";
	}

	String eventName(Element element) => '${blocName(element)}Event';

	String blocName(Element element) => element.name;

	String mapEventsToState(Element element, Set<MethodElement> events, final String stateName) {
		return '''
    @override
    Stream<$stateName> mapEventToState(final ${eventName(element)} event) async* {
        ${events.map((event) => 'if (event is ${className(event.name)}) yield* ${event.name}(${event.parameters.map((parameter) => 'event.${parameter.name}').join(', ')});').join('\n')}
    }
    ''';
	}

	mapEventToState(Element element, MethodElement event, final String stateName) {
		return 'Stream<$stateName> ${event.name}(${event.parameters.map((parameter) => 'final ${parameter.type} ${parameter.name}').join(', ')});';
	}
}
