import 'dart:async';

import 'package:analyzer/dart/element/element.dart';
import 'package:b_annotations/bloc.dart';
import 'package:b_generator/src/helpers/string_helpers.dart';
import 'package:b_generator/src/helpers/to_string.dart';
import 'package:build/build.dart';
import 'package:source_gen/source_gen.dart';

import 'helpers/annotated_method_visitor.dart';
import 'helpers/unnamed_constructor.dart';

class EventGenerator extends GeneratorForAnnotation<BlocAnnotation> {
  @override
  FutureOr<String> generateForAnnotatedElement(final Element element, final ConstantReader annotation, final BuildStep buildStep) {
    final Set<MethodElement> events = {};
    element.visitChildren(AnnotatedMethodVisitor<EventAnnotation>((method) => events.add(method)));
    return """
    class ${element.name}Event {}
    
    ${events.map((method) => createEvent(element, method)).join('\n')}
    """;
  }

  String createEvent(final Element element, final MethodElement method) {
    return '''class ${className(method.name)} extends ${element.name}Event {
      ${createEventFields(method.parameters)}

      ${linkedConstructorByString(className(method.name), method.parameters)}
      
      ${toString(method.parameters)}
    }''';
  }

  String createEventFields(final List<ParameterElement> parameters) {
    return parameters.map((p) => 'final ${p.type} ${p.name};').join('\n\t');
  }
}
