import 'dart:async';

import 'package:analyzer/dart/element/element.dart';
import 'package:b_annotations/state.dart';
import 'package:build/build.dart';
import 'package:source_gen/source_gen.dart';

import 'helpers/field_getters.dart';
import 'helpers/field_visitor.dart';
import 'helpers/parameters.dart';
import 'helpers/props.dart';

class StateGenerator extends GeneratorForAnnotation<StateAnnotation> {
  @override
  FutureOr<String> generateForAnnotatedElement(Element element, ConstantReader annotation, BuildStep buildStep) {
    final Set<FieldElement> fields = {};
    element.visitChildren(FieldVisitor((field) => fields.add(field)));

    return """
    abstract class GeneratedState extends Equatable {
      ${fieldGetters(fields)}
      
      ${copyWith(element, fields)}
      
      ${props(fields)}
    }""";
  }

  String copyWith(final Element element, final Iterable<FieldElement> fields) {
    return """${element.name} copyWith({
		${parameterSignature(fields)}
	}) {
		return ${element.name}(
			${fields.map(fieldToInstantiation).join(',\n')}
		);	
	}""";
  }

  String fieldToInstantiation(final FieldElement field) {
    return "${field.name}: ${field.name} ?? this.${field.name}";
  }
}
